@@ -29,17 +29,14 @@ static GList *
 location_get_cities (GWeatherLocation *parent_location)
 {
         GList *cities = NULL;
-        GWeatherLocation **children;
-        gint i;
+        GWeatherLocation *child = NULL;
 
-        children = gweather_location_get_children (parent_location);
-        for (i = 0; children[i]; i++) {
-                if (gweather_location_get_level (children[i]) == GWEATHER_LOCATION_CITY) {
-                        cities = g_list_prepend (cities,
-                                                 children[i]);
+        while ((child = gweather_location_next_child (parent_location, child))) {
+                if (gweather_location_get_level (child) == GWEATHER_LOCATION_CITY) {
+                        cities = g_list_prepend (cities, gweather_location_ref (child));
                 } else {
                         cities = g_list_concat (cities,
-                                                location_get_cities (children[i]));
+                                                location_get_cities (child));
                 }
         }
 
@@ -101,10 +98,10 @@ load_timezones (GList *cities)
 GList *
 weather_tz_db_get_locations (const gchar *country_code)
 {
-        GList *cities;
+        g_autoptr(GWeatherLocation) world = NULL;
+        g_autoptr(GWeatherLocation) country = NULL;
+        g_autolist(GWeatherLocation) cities = NULL;
         GList *tz_locations;
-        GWeatherLocation *world;
-        GWeatherLocation *country;
 
         world = gweather_location_get_world ();
 
@@ -116,7 +113,5 @@ weather_tz_db_get_locations (const gchar *country_code)
         cities = location_get_cities (country);
         tz_locations = load_timezones (cities);
 
-        g_list_free (cities);
-
         return tz_locations;
 }
